==> windowmatrixd-0.1.1.tar.gz: windowmatrixd-0.1.1/devices.c <==
#include <linux/bmc_pio.h>

#include <sys/ioctl.h>

#include <unistd.h>
#include <fcntl.h>

#define __USE_GNU 1
#define _GNU_SOURCE 1
#include <stdlib.h>
#include <stdio.h>

#include "config.h"
#include "devices.h"


/* setup_devices
This function initializes the output devices. */

int *
setup_devices (void)
{
  int  *  devices, i;
  char *  devicename, c;

  if ((devices = (int *) malloc (sizeof (int) * NUMBER_OF_DEVS)))
    {
      for (i = 0; (i < NUMBER_OF_DEVS) && devices; i++)
	devices[i] = -1;
      for (i = 0; (i < NUMBER_OF_DEVS) && devices; i++)
	{
	  asprintf (&devicename, "/dev/pio%i", i);
	  if ((devices[i] = open (devicename, O_RDWR)) != -1)
	    {
	      c = (char) BMC_PIO_OUTPUT;
	      if (!(ioctl (devices[i], BMC_PIO_CTL_DIRECTION, c)))
		{
		  c = (char) BMC_PIO_GROUPA_M0;
		  if (!(ioctl (devices[i], BMC_PIO_CTL_SETMODE_A, c)))
		    {
		      c = (char) BMC_PIO_GROUPB_M0;
		      if (!(ioctl (devices[i], BMC_PIO_CTL_SETMODE_B, c)))
			continue;
		    }
		}
	    }
	  close_devices (devices);
	  devices = NULL;
	  break;
	}
    }
  return (devices);
}


/* close_devices
This function closes the output devices. */

void
close_devices (int *  devices)
{
  int i;
  for (i = 0; (i < NUMBER_OF_DEVS) && (devices[i] != -1); i++)
    {
      close (devices[i]);
    }
  free (devices);
}


/* output_frame
This function "draws" the frame on the Windows. */

inline void
output_frame (const frame_t *  frame, const int *  devices)
{
  int           x, y;
  unsigned char output;

  printf("\033[2J\033[H");
  for (y = 0; y < HEIGHT; y++)
    {
      for (x = 0; x < WIDTH; x++)
	{
	  (frame->frame_data[x][y]) ? printf("#") : printf(" ");
	}
      printf("\n");
    }

  for (output = y = 0; y < (WIDTH * HEIGHT); y++)
    {
      x = y % 8;
      if ((!x) && (y))
	{
	  write (devices[(y / 8) - 1], &output, 1);
	  output = 0;
	}
      if (frame->frame_data[y])
	{
	  output += 1 << x;
	}
    }
  if (x)
    write (devices[(y / 8) - 1], &output, 1);
}

==> windowmatrixd-0.1.2.tar.gz: windowmatrixd-0.1.2/devices.c <==
#include <linux/bmc_pio.h>

#include <sys/ioctl.h>

#include <unistd.h>
#include <fcntl.h>

#include <stdlib.h>
#include <stdio.h>

#include "config.h"
#include "devices.h"


/* setup_devices
This function initializes the output devices. */

int *
setup_devices (void)
{
  int  *  devices, i;
  char *  devicename, c;

  if ((devices = (int *) malloc (sizeof (int) * NUMBER_OF_DEVS)))
    {
      for (i = 0; (i < NUMBER_OF_DEVS) && devices; i++)
	devices[i] = -1;
      for (i = 0; (i < NUMBER_OF_DEVS) && devices; i++)
	{
	  asprintf (&devicename, "/dev/pio%i", i);
	  if ((devices[i] = open (devicename, O_RDWR)) != -1)
	    {
	      c = (char) BMC_PIO_OUTPUT;
	      if (!(ioctl (devices[i], BMC_PIO_CTL_DIRECTION, &c)))
		{
		  c = (char) BMC_PIO_GROUPA_M0;
		  if (!(ioctl (devices[i], BMC_PIO_CTL_SETMODE_A, &c)))
		    {
		      c = (char) BMC_PIO_GROUPB_M0;
		      if (!(ioctl (devices[i], BMC_PIO_CTL_SETMODE_B, &c)))
			continue;
		    }
		}
	    }
	  free (devicename);
	  close_devices (devices);
	  devices = NULL;
	  break;
	}
    }
  return (devices);
}


/* close_devices
This function closes the output devices. */

void
close_devices (int *  devices)
{
  int i;
  for (i = 0; (i < NUMBER_OF_DEVS) && (devices[i] != -1); i++)
    {
      close (devices[i]);
    }
  free (devices);
}


/* output_frame
This function "draws" the frame on the Windows. */

inline void
output_frame (const frame_t *  frame, const int *  devices)
{
  int           x, y;
  unsigned char output;

  printf("\033[2J\033[H");
  for (y = 0; y < HEIGHT; y++)
    {
      for (x = 0; x < WIDTH; x++)
	{
	  (frame->frame_data[y][x]) ? printf("#") : printf(" ");
	}
      printf("\n");
    }

  for (output = y = 0; y < (WIDTH * HEIGHT); y++)
    {
      x = y % 8;
      if ((!x) && (y))
	{
	  write (devices[(y / 8) - 1], &output, 1);
	  output = 0;
	}
      if (((char *) frame->frame_data)[y] != '\0')
	{
	  output += 1 << x;
	}
    }
  if (x)
    write (devices[(y / 8) - 1], &output, 1);
}
